"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TimerState = exports.Interval = exports.Timeout = void 0;
var Timeout = /** @class */ (function () {
    function Timeout(callback, timeMS, autoStart) {
        var _this = this;
        if (typeof callback == "number") {
            autoStart = !!timeMS;
            timeMS = callback;
            (this._callbacks = []).push(function () { return _this.state = 3; });
        }
        else {
            (this._callbacks = []).push(function () { _this.state = 3; callback.call(_this); });
        }
        this.currentTime = Number(timeMS);
        this._startedAt = 0;
        this._timeLeft = this.currentTime;
        this._timerId = null;
        this.state = 0;
        if (autoStart)
            this.start();
    }
    Object.defineProperty(Timeout.prototype, "timeLeft", {
        get: function () {
            if (this.state == 0 || this.state == 3)
                return 0;
            return Date.now() - this._startedAt;
        },
        enumerable: false,
        configurable: true
    });
    Timeout.prototype.start = function (timeMS) {
        if (this.state == 3)
            this.stop();
        if (arguments.length > 0 && this.state == 0)
            this.currentTime = timeMS;
        if (this._timerId == null) {
            if (this.state == 2) {
                this._startedAt = Date.now();
                this._timerId = setTimeout(this._executeCallbacks.bind(this), this._timeLeft);
                this.state = 1;
            }
            else if (this.state == 0) {
                this._startedAt = Date.now();
                this._timerId = setTimeout(this._executeCallbacks.bind(this), this.currentTime);
                this.state = 1;
            }
        }
        return this;
    };
    Timeout.prototype.pause = function () {
        if (this.state != 1)
            return this;
        clearTimeout(this._timerId);
        this._timerId = null;
        this._timeLeft -= Date.now() - this._startedAt;
        this.state = 2;
        return this;
    };
    Timeout.prototype.stop = function () {
        return this._stop(true);
    };
    Timeout.prototype._stop = function (reject) {
        clearTimeout(this._timerId);
        this._timerId = null;
        this._timeLeft = 0;
        this.state = 0;
        if (reject)
            this._rejecters.splice(0).forEach(function (r) { return r(); });
        return this;
    };
    Timeout.prototype.restart = function () {
        return this._stop(false).start();
    };
    Timeout.prototype.then = function (onResolve, onReject) {
        this._callbacks.push(onResolve);
        this._rejecters.push(onReject);
    };
    Timeout.prototype._executeCallbacks = function () {
        var _this = this;
        this._callbacks.forEach(function (c) { return c.call(_this); });
        this._callbacks = [this._callbacks[0]];
    };
    return Timeout;
}());
exports.Timeout = Timeout;
var Interval = /** @class */ (function () {
    function Interval(callback, timeMS, autoStart) {
        var _this = this;
        this._callback = function () { _this._lastTrigger = Date.now(); callback.call(_this); };
        this.currentTime = timeMS;
        this._lastTrigger = 0;
        this._timeLeft = timeMS;
        this._timerId = null;
        this.state = 0;
        this._isInTimeout = false;
        if (autoStart)
            this.start();
    }
    Object.defineProperty(Interval.prototype, "timeLeft", {
        get: function () {
            if (this.state == 0)
                return 0;
            return Date.now() - this._lastTrigger;
        },
        enumerable: false,
        configurable: true
    });
    Interval.prototype.start = function (timeMS) {
        var _this = this;
        if (arguments.length > 0 && this.state == 0)
            this.currentTime = timeMS;
        if (this._timerId == null) {
            if (this.state == 2) {
                this._lastTrigger = Date.now();
                this._timerId = setTimeout(function () {
                    _this._isInTimeout = false;
                    _this._timerId = setInterval(_this._callback, _this.currentTime);
                }, this._timeLeft);
                this.state = 1;
                this._isInTimeout = true;
            }
            else if (this.state == 0) {
                this._lastTrigger = Date.now();
                this._timerId = setInterval(this._callback, this.currentTime);
                this.state = 1;
            }
        }
        return this;
    };
    Interval.prototype.pause = function () {
        if (this.state != 1)
            return this;
        this._isInTimeout ? clearTimeout(this._timerId) : clearInterval(this._timerId);
        this._timerId = null;
        this._timeLeft -= Date.now() - this._lastTrigger;
        this.state = 2;
        return this;
    };
    Interval.prototype.stop = function () {
        this._isInTimeout ? clearTimeout(this._timerId) : clearInterval(this._timerId);
        this._timerId = null;
        this._timeLeft = 0;
        this.state = 0;
        this._isInTimeout = false;
        return this;
    };
    return Interval;
}());
exports.Interval = Interval;
var TimerState;
(function (TimerState) {
    TimerState[TimerState["Reset"] = 0] = "Reset";
    TimerState[TimerState["Running"] = 1] = "Running";
    TimerState[TimerState["Paused"] = 2] = "Paused";
    TimerState[TimerState["Done"] = 3] = "Done";
})(TimerState = exports.TimerState || (exports.TimerState = {}));
